.g-item-header
  border-bottom 1px solid #ddd
  margin-bottom 12px
  padding-bottom 8px
  font-size 16px
  color #555

.g-item-edit-button
  margin-bottom 2px
  display inline
  font-size 14px
  float right

.g-item-files, .g-item-metadata
  margin-top 20px

.g-item-files-header, .g-item-info-header
  background-color #f0f0f0
  padding 5px 6px
  color #555
  font-size 16px
  font-weight bold

.g-info-list-entry
  margin 3px
  margin-top 5px
  background-color #fbfbfb

.editing
  background-color #C6DEFF

.g-file-list
  list-style-type none
  padding-left 0
  margin-bottom 0

  >li
    padding 3px 5px
    border-bottom 1px solid #f2f2f2

    .g-file-list-link
      margin-right 3px
      border-right 1px solid #e3e3e3
      padding-right 6px

    .g-file-size
      margin 0 6px
      color #888
      display inline-block

    .g-file-actions-container
      float right

.g-item-breadcrumb-container
  margin-bottom 10px

  ol.breadcrumb
    margin-bottom 0
    background-color #eaebea
    border-radius 0
    padding-left 5px
    padding-right 5px

  .g-hierarchy-level-up
    border 1px solid #d5d5d5
    border-radius 3px
    padding 2px 3px

    &:hover
      border 1px solid #aaa
      background-color #dadada

.g-item-actions-button
  margin-left 10px !important

.g-right-border
  margin-right 3px
  border-right 1px solid #e3e3e3
  padding-right 6px
