#g-app-footer-container
  border-top 1px solid #eee
  text-align center
  background-image linear-gradient(to bottom, #f6f6f6 0, #fff 5px)

.g-quick-search-form
  >.form-group
    display inline-block !important
    margin-bottom 0 !important
  .form-control
    display inline !important

.g-footer-info
  margin-bottom 10px

.g-footer-links
  margin 15px auto 10px
  a
    margin 0 10px
